class RubygemsController < ApplicationController
  unloadable
  
  before_filter :find_project, :authorize, :only => [ :index, :create ]
  before_filter :find_repository

  
  
  def index
    @versions = @project.versions
    @rubygems = Rubygem.find(:all, :conditions => "project_id = #{@project.id}")
  end

  def create
    gemdir_path = RubygemsProjectSetting.find_or_create(@project.id).gemdir_path
    export_path = RubygemsProjectSetting.find_or_create(@project.id).export_path
    if User.current.admin? || (@project && User.current.allowed_to?(:build_rubygems, @project)) then
    mes = "- " + l('rubygems.controller.removing_old_export_path') + "..."
    if File::exist?("#{export_path}/#{params[:tags]}") and !params[:tags].empty? then
      if FileUtils::remove_dir("#{export_path}/#{params[:tags]}") then
        mes << "[OK]<br>"
      else  
        mes << "[KO]<br>"
        flash[:error] = l('rubygems.controller.building_failed') + " : <br>" + mes
        redirect_to :controller => 'rubygems', :action => 'index', :project_id => @project
        return
      end
    else
      mes << "[N/A]<br>"
    end
    
    mes << "- " + l('rubygems.controller.svn_export') + "..."
    `cd #{export_path} && /usr/bin/env svn export #{@repository.url}/tags/#{params[:tags]}`
    if $?.to_i == 0 then
      mes << "[OK]<br>"
      mes << " * " + l('rubygems.controller.get_methods') + " : #{params[:methods]}<br>"
      if params[:methods] == 'Rakefile' then 
        mes << "- " + l('rubygems.controller.build_with', :method => params[:methods] ) + "..."
        `cd #{export_path}/#{params[:tags]}; /usr/bin/env rake clean clobber gem`
      else
        mes << "- " + l('rubygems.controller.build_with', :method => "Gemspec : #{params[:methods]}" ) + "..."
        `cd #{export_path}/#{params[:tags]}; /usr/bin/env gem build #{params[methods]} `
      end
      if $?.to_i == 0 then
        mes << "[OK]<br>"
        filename = Dir["#{export_path}/#{params[:tags]}/pkg/*-#{params[:version]}.gem"].first

        mes << "- " + l('rubygems.controller.deploy_local') + "..."
        if User.current.admin? || (@project && User.current.allowed_to?(:local_deployment_of_rubygems, @project)) then
          unless params[:local_deployment].nil? then
            `/usr/bin/env gem ins #{filename}`
            if $?.to_i == 0 then
              mes << "[OK]<br>"
            else
              mes << "[KO]<br>"
              flash[:error] = l('rubygems.controller.building_failed') + " : <br>" + mes
              redirect_to :controller => 'rubygems', :action => 'index', :project_id => @project
              return
            end
          else
            mes << "[N/A]<br>"
          end
        else
          mes << "[LOCK]<br>"
        end
        
        mes << "- " + l('rubygems.controller.deploy_gemdir') + "..."
        if User.current.admin? || (@project && User.current.allowed_to?(:server_deployment_of_rubygems, @project)) then
          unless params[:server_deployment].nil? then
            `/usr/bin/env gem ins -i #{gemdir_path} -n /tmp #{filename}`
            if $?.to_i == 0 then
              mes << "[OK]<br>"
            else
              mes << "[KO]<br>"
              flash[:error] = l('rubygems.controller.building_failed') + " : <br>" + mes
              redirect_to :controller => 'rubygems', :action => 'index', :project_id => @project
              return
            end
          else
            mes << "[N/A]<br>"
          end
        else
          mes << "[LOCK]<br>"
        end

        mes << "- " + l('rubygems.controller.deploy_gemcutter') + "..."
        if User.current.admin? || (@project && User.current.allowed_to?(:gemcutter_deployment_of_rubygems, @project)) then
          unless params[:gemcutter_deployment].nil? then
            `/usr/bin/env gem push #{filename}`
            if $?.to_i == 0 then
              mes << "[OK]<br>"
            else
              mes << "[KO]<br>"
              flash[:error] = l('rubygems.controller.building_failed') + " : <br>" + mes
              redirect_to :controller => 'rubygems', :action => 'index', :project_id => @project
              return
            end
          else
            mes << "[N/A]<br>"
          end
        else
          mes << "[LOCK]<br>"
        end

        fBaseName = File::basename(filename)
        mes << "- " + l('rubygems.controller.removing_old_attachment') + "..."
        if Gemfile.delete_all("filename = '#{fBaseName}'") then
          mes << "[OK]<br>"
        else
          mes << "[N/A]<br>"
        end
        mes << "- " + l('rubygems.controller.attachment_in_redmine', :gemfile => fBaseName ) + "..."
        version = Version.find(:first, :conditions => "name = #{params[:version]} and project_id = #{@project.id}")        
        if Gemfile.create(:container => version, 
                              :file => filename,
                              :description => "#{params[:comments]}",
                              :author => User.current ) then
          mes << "[OK]<br>"
          unless params[:rebuild] then
            mes << "- " + l('rubygems.controller.create_record_in_redmine', :gemfile => fBaseName ) + "..."
            @rubygem = Rubygem.new(:project => @project, :builder => User.current, :version => version )

            @rubygem.filename = fBaseName
            @rubygem.svn_tags = params[:tags]
          else
            mes << "- " + l('rubygems.controller.modify_record_in_redmine', :gemfile => fBaseName ) + "..."
            @rubygem = Rubygem.find(:first, :conditions => "svn_tags = #{params[:tags]} and project_id = #{@project.id}")
          end
          @rubygem.comments = params[:comments]
            
          
          if @rubygem.save
              mes << "[OK]<br>"
              mes << "- " + l('rubygems.controller.create_news_in_redmine') + "..."
              unless params[:annonce_enabled].nil?
                @news = News.new(:project => @project, :author => User.current)
                @news.attributes = { :title => l('rubygems.models.news.title', :gemfile => fBaseName ),
                                     :summary => l('rubygems.models.news.summary'),
                                     :description => l('rubygems.models.news.description', :version => version)
                                     }
                if @news.save
                  mes << "[OK]<br>"
                else
                  mes << "[KO]<br>"
                  flash[:error] = l('rubygems.controller.building_failed') + " : <br>" + mes
                  redirect_to :controller => 'rubygems', :action => 'index', :project_id => @project
                end
              else
                mes << "[N/A]<br>"
              end

            

            flash[:notice] = l('rubygems.controller.building_success') + " : <br>" + mes  
          else
            mes << "[KO]<br>"
            flash[:error] = l('rubygems.controller.building_failed') + " : <br>" + mes 
          end
        else
          mes << "[KO]<br>"
          flash[:error] = l('rubygems.controller.building_failed') + " : <br>" + mes 
        end                       
      else
          mes << "[KO]<br>"
          flash[:error] = l('rubygems.controller.building_failed') + " : <br>" + mes 
      end
    else
       mes << "[KO]<br>"
      flash[:error] = l('rubygems.controller.building_failed') + " : <br>" + mes 
    end
      
    else
      flash[:error] = l('rubygems.controller.building_refused')
    end
    redirect_to :controller => 'rubygems', :action => 'index', :project_id => @project
    return
  end


  


  
  private
        def find_project
          @project = Project.find(params[:project_id])
        rescue ActiveRecord::RecordNotFound
          render_404
        end
    


    def find_repository
      @project = Project.find(params[:project_id])
      @repository = @project.repository
      @path = 'tags'
      @rev = ''
      unless @repository.nil? then
        @entries = @repository.entries(@path, @rev)
        @buildable= Hash::new  
        if @entries then
          @entries.each do |arelease|
            @allfiles = @repository.entries("#{@path}/#{arelease.name}", @rev)
            files = Array::new
            @allfiles.each do |file|
              files.push file.name
            end
            build = []
            files.each  {|v| 
              build.push v if v=~ /Rakefile/
              build.push v if v =~  /.*\.gemspec/
            }   
            @buildable[arelease.name] = build
            @buildable.delete_if {|key, value| value.empty? }
          end  
        end
      end
    end
end
  